﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Runtime.Serialization.Formatters.Binary;
using System.Diagnostics;

namespace Match_Snatch
{
    [Serializable]
    public class CMatches : INotifyPropertyChanged
    {
        #region data binding

        [field: NonSerialized]
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string pn)
        {
            if(PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(pn));
        }
        #endregion

        #region fields and their properties
        private int total;

        public int Total
        {
            get { return total; }
            set
            {
                total = value;
                OnPropertyChanged("total");
            }
        }
        private int limit;

        public int Limit
        {
            get { return limit; }
            set
            {
                limit = value;
                OnPropertyChanged("limit");
            }
        }
        private bool playerGo;

        public bool PlayerGo
        {
            get { return playerGo; }
            set
            {
                playerGo = value;
                OnPropertyChanged("playerGo");
            }
        }
        #endregion

        #region constructors
        public CMatches(int newTotal, int newLimit)
        {
            total = newTotal;
            limit = newLimit;
            playerGo = true;
        }

        public CMatches()
        {
            total = 30;
            limit = 7;
            playerGo = true;
        }
        #endregion

        #region methods
        public void subtract(int numTaken)
        {
            // an assert statement to catch errors
            Trace.Assert((numTaken > 0) && (numTaken <= limit), "Number of matches taken out of bounds.");

            total -= numTaken;

            // prevents the total going below zero
            if (total < 0)
                total = 0;

            // enforces the data binding
            OnPropertyChanged("total");
        }

        public void save(string fileName)
        {
            // an assert statement to catch errors
            Trace.Assert(fileName.Length > 0, "Invalid filename.");

            // creates the necessary objects to save the object in the correct format
            FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate);
            BinaryFormatter bf = new BinaryFormatter();

            // serializes the data
            bf.Serialize(fs, this);

            // closes the stream to prevent data loss
            fs.Close();
        }

        public static CMatches load(string fileName)
        {
            // an assert statement to catch errors
            Trace.Assert(fileName.Length > 0, "Invalid filename.");

            // creates the objects necessary to load a class
            FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read);
            BinaryFormatter bf = new BinaryFormatter();

            // deserialises the class
            CMatches cVar = (CMatches)bf.Deserialize(fs);

            // closes the stream
            fs.Close();

            // returns the deserialised class
            return cVar;
       }

        public bool isWon()
        {
            return (total == 0);
        }
        #endregion
    }
}
