﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Match_Snatch
{
    public partial class frmMain : Form
    {
        // global objects to be used throughout the program
        CMatches game;
        CComputerPlayer ai;
        Graphics g;
        int xOffset = 0;
        int yOffset = 0;
        const int DEFAULT_NUM_OF_MATCH_YSECTIONS = 20;
        int numOfMatchYSections = DEFAULT_NUM_OF_MATCH_YSECTIONS;

        public frmMain()
        {
            InitializeComponent();

            // initialises the global objects
            game = new CMatches(Properties.Settings.Default.DefaultTotal, Properties.Settings.Default.DefaultLimit);
            ai = new CComputerPlayer();
            g = pnlGUI.CreateGraphics();

            // setting up data bindings
            setUpDataBindings();
            resetGUI();
            lblMatchesToTake.DataBindings.Add("Text", tbPlayerMove, "Value", true);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("Are you sure you wish to exit?", "Closing", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
            {
                e.Cancel = true;
            }
            else
            {
                // saves all the changes to the properties
                Properties.Settings.Default.Save();
            }
        }

        // opens a previously saved game
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = openFileDialog1.ShowDialog();

            if (dr == DialogResult.OK)
            {
                try
                {
                    breakDataBindings();

                    // changes the current game to a previously saved one
                    game = CMatches.load(openFileDialog1.FileName);

                    setUpDataBindings();

                    // enforces data binding
                    game.OnPropertyChanged("total");
                    game.OnPropertyChanged("limit");

                    // resets the tracker bar
                    resetTrackerBar();
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.Message + "\nCannot load game correctly.", "Saving error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            if (game.PlayerGo)
            {
                btnTakeMatches.Text = "Take player's turn";
            }
            else
            {
                btnTakeMatches.Text = "Take computer's turn";
            }
        }

        // saves the current game
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = saveFileDialog1.ShowDialog();

            if (dr == DialogResult.OK)
            {
                try
                {
                    // saves the current game
                    game.save(saveFileDialog1.FileName);
                }
                catch (Exception exc)
                {
                    MessageBox.Show(exc.Message + "\nCannot save game correctly.", "Saving error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }                
            }
        }

        private void Form1_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            // sets up the F1 key to display a compiled help file
            // but if the mouse is clicked, then a message box is displayed
            if (Control.MouseButtons == MouseButtons.None)
            {
                getHelp();
            }
            else
            {
                Point p = PointToClient(Cursor.Position);
                Control c = GetChildAtPoint(p);

                // if no control is clicked, then the function exits
                if (c == null)
                    return;

                // creates a string to output the help message
                string helpStr = toolTip1.GetToolTip(c);

                if (string.IsNullOrEmpty(helpStr))
                    helpStr = "No help is available for this control.";

                // displays the relevant message to the user and
                // sets the event to handled
                MessageBox.Show(helpStr, "Help");
                hlpevent.Handled = true;
            }
        }

        private void changeCurrentSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmSettings f2 = new frmSettings();

            // sets up the settings form before it is displayed
            f2.Total = game.Total;
            f2.Limit = game.Limit;

            DialogResult dr = f2.ShowDialog();

            if (dr == DialogResult.OK)
            {
                game.Total = f2.Total;
                game.Limit = f2.Limit;
                tbPlayerMove.Maximum = f2.Limit;
                resetGUI();

                // refreshs the GUI panel
                pnlGUI.Refresh();
            }
        }

        private void changeDefaultSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmSettings f2 = new frmSettings();

            // setting up the settings form before it is displayed
            f2.Total = Properties.Settings.Default.DefaultTotal;
            f2.Limit = Properties.Settings.Default.DefaultLimit;

            DialogResult dr = f2.ShowDialog();

            if (dr == DialogResult.OK)
            {
                Properties.Settings.Default.DefaultTotal = f2.Total;
                Properties.Settings.Default.DefaultLimit = f2.Limit;
                resetGUI();

                // refreshs the GUI panel
                pnlGUI.Refresh();
            }
        }

        private void btnTakeMatches_Click(object sender, EventArgs e)
        {
            // prevents the user from changing the settings mid game
            changeCurrentSettingsToolStripMenuItem.Enabled = false;

            
            if (game.PlayerGo)
            {
                // takes the players move
                game.subtract((int)tbPlayerMove.Value);
                btnTakeMatches.Text = "Take computer's turn";
            }
            else
            {
                // takes the computers move
                game.subtract(ai.calcMove(game.Total, game.Limit));
                btnTakeMatches.Text = "Take player's turn";
            }

            // changes turn
            game.PlayerGo = !game.PlayerGo;

            // checks to see if the game has been won
            if(game.isWon())
            {
                // shows the appropriate message at the end of the game
                if(game.PlayerGo)
                {
                    MessageBox.Show("You have won!", "Congratulations", MessageBoxButtons.OK);
                }
                else
                {
                    MessageBox.Show("The computer has won.", "Commisserations", MessageBoxButtons.OK);
                }

                // stores the result of the next dialog
                DialogResult dr = MessageBox.Show("Do you wish to play again?", "Play Again?", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);

                // starts a new game or exits depending on user input
                if(dr == DialogResult.Yes)
                {
                    breakDataBindings();

                    game = new CMatches(Properties.Settings.Default.DefaultTotal, Properties.Settings.Default.DefaultLimit);

                    setUpDataBindings();

                    // allows the user to change settings before the game begins
                    changeCurrentSettingsToolStripMenuItem.Enabled = true;
                }
                else
                {
                    Application.Exit();
                }
            }

            // resets the tracker bar
            resetTrackerBar();

            // refreshs the GUI panel
            pnlGUI.Refresh();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            breakDataBindings();

            game = new CMatches(Properties.Settings.Default.DefaultTotal, Properties.Settings.Default.DefaultLimit);

            setUpDataBindings();

            // enforces data binding
            game.OnPropertyChanged("total");
            game.OnPropertyChanged("limit");

            // allows the user to change settings before the game begins
            changeCurrentSettingsToolStripMenuItem.Enabled = true;

            // resets the tracker bar
            resetTrackerBar();

            // refreshs the GUI panel
            pnlGUI.Refresh();

            if (game.PlayerGo)
            {
                btnTakeMatches.Text = "Take player's turn";
            }
            else
            {
                btnTakeMatches.Text = "Take computer's turn";
            }
        }

        #region functions to take care of data binding
        public void setUpDataBindings()
        {
            // setting up data bindings
            lblMatchesRemaining.DataBindings.Add("Text", game, "Total", true);
            lblCurrentLimit.DataBindings.Add("Text", game, "Limit", true);

            resetGUI();
        }

        public void breakDataBindings()
        {
            // removes data bindings
            lblCurrentLimit.DataBindings.Clear();
            lblMatchesRemaining.DataBindings.Clear();
        }
        #endregion

        public void getHelp()
        {
            // finds the help file and stores it's location
            string helpFile = Path.GetFullPath("htmlHelp.chm");

            // displays the compiled help file
            Help.ShowHelp(this, helpFile, HelpNavigator.Index);
        }

        public void resetTrackerBar()
        {
            // resets the maximum of the tracker bar
            if (game.Total < game.Limit)
            {
                tbPlayerMove.Maximum = game.Total;
            }
            else
            {
                tbPlayerMove.Maximum = game.Limit;
            }

            // resets the data binding for the appropriate label
            lblMatchesToTake.DataBindings.Clear();
            lblMatchesToTake.DataBindings.Add("Text", tbPlayerMove, "Value");
        }

        private void showHelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            getHelp();
        }

        private void pnlGUI_Paint(object sender, PaintEventArgs e)
        {
            // creates the brush objects in the relevant colour
            Brush matchHead = new SolidBrush(Color.Red);
            int headHeight = (int) (((numOfMatchYSections - 2) * 0.1) * yOffset);
            Brush matchBody = new SolidBrush(Color.Yellow);
            int bodyHeight = (int) (((numOfMatchYSections - 2) * 0.9) * yOffset);

            // sets the starting point to draw
            Point currentDrawingPoint = new Point(xOffset, yOffset);

            // draws as many matches as there are remaining
            for (int i = 0; i < game.Total; i++)
            {
                // draws the head of the match
                g.FillRectangle(matchHead, currentDrawingPoint.X, currentDrawingPoint.Y, xOffset, headHeight);

                // moves the current drawing point
                currentDrawingPoint.Y += headHeight;

                // draws the body of the match
                g.FillRectangle(matchBody, currentDrawingPoint.X, currentDrawingPoint.Y, xOffset, bodyHeight);

                // moves the current drawing point to the start of the next match
                currentDrawingPoint.X += (xOffset * 2);
                currentDrawingPoint.Y -= headHeight;
            }
        }

        private void resetGUI()
        {
            xOffset = pnlGUI.Size.Width / ((game.Total * 2) + 1);
            yOffset = (int)(pnlGUI.Size.Height * (1.0 / numOfMatchYSections));
        }
    }
}
